#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>    
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>	
    [Description("")]
    public class ImpBollinger : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int length = 20; // Default setting for Length
            private int devs = 2; // Default setting for Devs
        // User defined variables (add any user defined variables below)
		private SMA avg;
		private ImpStdDev sDev;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "MidLine"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Upper"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Lower"));
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			avg = SMA(Input,length);
			sDev = ImpStdDev(1,length);
		}

        protected override void OnBarUpdate()
        {
            MidLine[0] = avg[0];
			Upper[0] = avg[0] + devs * sDev[0];
			Lower[0] = avg[0] - devs * sDev[0];
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MidLine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Upper
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Lower
        {
            get { return Values[2]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Devs
        {
            get { return devs; }
            set { devs = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ImpBollinger[] cacheImpBollinger = null;

        private static ImpBollinger checkImpBollinger = new ImpBollinger();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public ImpBollinger ImpBollinger(int devs, int length)
        {
            return ImpBollinger(Input, devs, length);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public ImpBollinger ImpBollinger(Data.IDataSeries input, int devs, int length)
        {
            if (cacheImpBollinger != null)
                for (int idx = 0; idx < cacheImpBollinger.Length; idx++)
                    if (cacheImpBollinger[idx].Devs == devs && cacheImpBollinger[idx].Length == length && cacheImpBollinger[idx].EqualsInput(input))
                        return cacheImpBollinger[idx];

            lock (checkImpBollinger)
            {
                checkImpBollinger.Devs = devs;
                devs = checkImpBollinger.Devs;
                checkImpBollinger.Length = length;
                length = checkImpBollinger.Length;

                if (cacheImpBollinger != null)
                    for (int idx = 0; idx < cacheImpBollinger.Length; idx++)
                        if (cacheImpBollinger[idx].Devs == devs && cacheImpBollinger[idx].Length == length && cacheImpBollinger[idx].EqualsInput(input))
                            return cacheImpBollinger[idx];

                ImpBollinger indicator = new ImpBollinger();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Devs = devs;
                indicator.Length = length;
                Indicators.Add(indicator);
                indicator.SetUp();

                ImpBollinger[] tmp = new ImpBollinger[cacheImpBollinger == null ? 1 : cacheImpBollinger.Length + 1];
                if (cacheImpBollinger != null)
                    cacheImpBollinger.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheImpBollinger = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpBollinger ImpBollinger(int devs, int length)
        {
            return _indicator.ImpBollinger(Input, devs, length);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpBollinger ImpBollinger(Data.IDataSeries input, int devs, int length)
        {
            return _indicator.ImpBollinger(input, devs, length);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpBollinger ImpBollinger(int devs, int length)
        {
            return _indicator.ImpBollinger(Input, devs, length);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpBollinger ImpBollinger(Data.IDataSeries input, int devs, int length)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ImpBollinger(input, devs, length);
        }
    }
}
#endregion
